import { Component, EventEmitter, OnChanges, OnDestroy, OnInit, Output } from '@angular/core';
import { Channel } from '../../../models/channel';
import { ChatService } from '../../../services/chat.service';
import { Subject, takeUntil } from 'rxjs';

@Component({
  selector: 'app-channel-list',
  standalone: false,
  templateUrl: './channel-list.component.html',
  styleUrl: './channel-list.component.scss'
})
export class ChannelListComponent implements OnInit, OnDestroy {
  @Output("select") selectEmitter: EventEmitter<Channel> = new EventEmitter<Channel>();

  public channels!: Channel[];
  public selectedChannel?: Channel;

  private destroy = new Subject<void>();

  constructor(private chatService: ChatService) { }

  ngOnInit() {
    this.chatService.ListChannels()
      .pipe(takeUntil(this.destroy))
      .subscribe(channels => {
        this.channels = channels;
        this.selectedChannel = this.channels[0];
      });
  }

  ngOnDestroy(): void {
    this.destroy.next();
    this.destroy.complete();
  }

  public Select(index: number): void {
    this.selectEmitter.emit(this.selectedChannel = this.channels[index]);
  }
}
