import { Component, Input, OnChanges, OnDestroy, OnInit } from '@angular/core';
import { Message } from '../../../models/message';
import { ChatService } from '../../../services/chat.service';
import { UserService } from '../../../services/user.service';
import { Channel } from '../../../models/channel';
import { Subscription } from 'rxjs';

@Component({
  selector: 'app-feed',
  standalone: false,
  templateUrl: './feed.component.html',
  styleUrl: './feed.component.scss'
})
export class FeedComponent implements OnChanges, OnDestroy {
  private readonly DEFAULT_CHANNEL_ID: number = 1;

  @Input('channel') public channel?: Channel;
  public messages: Message[] = [];

  public subscription?: Subscription;

  constructor(private chatService: ChatService) { }

  ngOnChanges() {
    if (this.subscription) {
      this.subscription.unsubscribe();
      this.messages = [];
    }

    this.subscription =
      this.chatService.GetMessages(this.channel?.id ?? this.DEFAULT_CHANNEL_ID)
        .subscribe(message => this.messages.push(message));
  }

  ngOnDestroy(): void {
    if (this.subscription) {
      this.subscription.unsubscribe();
    }
  }
}
